require 'vr/vruby'
require 'vr/vrdialog'
require 'vr/vrmargin'
require 'nono/version'
require 'nono/gui/comvnono'
require 'nono/win/config'
require 'nono/win/vplate'

module Nono
  module Win
    class VNono < VRForm
      include VRMenuUseable
      include Nono::Gui::COMVNono
      SS_CENTER = 1
      def construct
        self.caption = "wnono"
        @file_menu = newMenu(true).set([["Open...", "open"], ["Quit", "quit"]])
        @view_menu = newMenu(true).set([["Check", "check"], 
                                       ["Answer", "answer"],
                                       ["Solve Step by Step", "solve_step"],
                                       ["Reset", "reset"]])
        @help_menu = newMenu(true).set([["About wnono", "about"]])
        setMenu newMenu.set([["File", @file_menu],
                            ["View", @view_menu],
                            ["Help", @help_menu]])

        font = @screen.factory.newfont(Config::FONTNAME, Config::FONTSIZE)
        @msg_height = font.height + 4
        addControl(VRStatic, "msg", "", 0, 0, 60, @msg_height, SS_CENTER).
                   extend(VRMargin).initMargin(0,0,0,nil)
        addControl(VPlate, "vplate", "").extend(VRMargin).initMargin(0,@msg_height,0,0)
        move(100, 100, 180, 60 + @msg_height + 10)
      end

      include VRResizeSensitive
      def self_resize(w, h)
        if @vplate
          @msg.move(0, 0, w, @msg_height)
          @vplate.move(0, 0, w, h)
        end
      end

      def open_file_dialog
        openFilenameDialog
      end

      def display_error_dialog(msg)
        messageBox(msg, "Error")
      end

      def resize(nw, nh)
        x, y, w, h = windowrect
        move(x, y, nw, nh)
      end

      def calc_width_from_plate
        width_from_plate + 10
      end

      def calc_height_from_plate
        height_from_plate + 60 + @msg_height
      end

      def quit_clicked
        self.close
      end

      def print_msg(msg)
        font = @screen.factory.newfont(Config::FONTNAME, Config::FONTSIZE)
        @msg.setFont(font)
        @msg.caption = msg
      end

      def about_clicked
        msg = "wnono #{Nono::VERSION}"
        msg += "\r\n" + "Ruby #{RUBY_VERSION} (#{RUBY_RELEASE_DATE}) [#{RUBY_PLATFORM}]"
        msg += "\r\n" + "VisualuRuby #{SWin::VERSION}"
        messageBox msg, "about wnono"
      end

    end
  end
end

